unit USendMail;

interface

uses
  Classes;

function SendMailHTTP(strHost, strPHPScript, strPassword: string; strFrom, strTo, strBCC, strSubject, strMessage: string; slAttachments: TStrings): boolean;

implementation

uses
  SysUtils, Dialogs,
  IdSMTP, IdMessage, IdAttachmentFile, IdGlobal, IdSSLOpenSSL, IdExplicitTLSClientServerBase,
  Idhttp, IdMultipartFormData, IdException;

// with the proper information you can also send a email by SMTP, a bit tricky for the user he
// has to provide a SMTP server, or you can use your own SMTP server, but they are
// not always available
procedure SendMailSMTP(strHostName, strSMTPUsername, strSMTPPassword: string; iSMTPPort: integer; strFrom, strTo, strBCC, strSubject, strMessage: string; slAttachments: TStrings);
var
  oSMTP: TIdSMTP;
  oMailMessage: TIdMessage;
  oIdSSLIOHandlerSocketOpenSSL: TIdSSLIOHandlerSocketOpenSSL;
  i: integer;
begin
  oIdSSLIOHandlerSocketOpenSSL := nil;
  oSMTP := TIdSMTP.Create;
  oMailMessage := TIdMessage.Create;
  try
    oSMTP.Host := strHostName;
    oSMTP.Port := iSMTPPort;
    oSMTP.AuthType := satDefault;
    if (iSMTPPort = 465) then begin
      oIdSSLIOHandlerSocketOpenSSL := TIdSSLIOHandlerSocketOpenSSL.Create;
      oSMTP.IOHandler := oIdSSLIOHandlerSocketOpenSSL;
      oSMTP.UseTLS := utUseExplicitTLS;
      oIdSSLIOHandlerSocketOpenSSL.Destination := strHostName + ':' + IntToStr(iSMTPPort);
      oIdSSLIOHandlerSocketOpenSSL.Host := 'smtp.gmail.com';
      oIdSSLIOHandlerSocketOpenSSL.MaxLineAction := maException;
      oIdSSLIOHandlerSocketOpenSSL.Port := iSMTPPort;
      oIdSSLIOHandlerSocketOpenSSL.SSLOptions.Method := sslvTLSv1;
      oIdSSLIOHandlerSocketOpenSSL.SSLOptions.Mode := sslmUnassigned;
      oIdSSLIOHandlerSocketOpenSSL.SSLOptions.VerifyMode := [];
      oIdSSLIOHandlerSocketOpenSSL.SSLOptions.VerifyDepth := 0;
    end;
    oSMTP.Username := strSMTPUsername;
    oSMTP.Password := strSMTPPassword;

    oMailMessage.From.Address := strFrom;
    oMailMessage.Recipients.EMailAddresses := strTo;
    oMailMessage.BccList.EMailAddresses := strBCC;
    oMailMessage.ReplyTo.EMailAddresses := strFrom;

    oMailMessage.Subject := strSubject;
    oMailMessage.Body.Text := strMessage;

    if Assigned(slAttachments) then begin
      for i := 0 to slAttachments.Count - 1 do
        if FileExists(slAttachments[i]) then
          TIdAttachmentFile.Create(oMailMessage.MessageParts, slAttachments[i]);
    end;

    try
      oSMTP.MailAgent := ExtractFileName(ParamStr(0));
      oSMTP.ConnectTimeout := 2000;
      oSMTP.Connect;
      oSMTP.Send(oMailMessage);
    finally
      if oSMTP.Connected then
        oSMTP.Disconnect;
    end;

  finally
    oIdSSLIOHandlerSocketOpenSSL.Free;
    oMailMessage.Free;
    oSMTP.Free;
  end;
end;

// Send email information to a (PHP) web server which will create and send a email
// provide a host, url, password together with the information for the email. This one allows 1
// attachment
function SendMailHTTP(strHost, strPHPScript, strPassword: string; strFrom, strTo, strBCC, strSubject, strMessage: string; slAttachments: TStrings): boolean;
var
  Response: string;
  HTTPClient: TidHTTP;
  FormDataStream: TIdMultipartFormDataStream;
begin
  Result := false;
  try
    HTTPClient := TidHTTP.Create;
    try
      FormDataStream := TIdMultiPartFormDataStream.Create;
      try
        FormDataStream.AddFormField('password', strPassword);
        FormDataStream.AddFormField('emailto', strTo);
        FormDataStream.AddFormField('emailfrom', strFrom);
        FormDataStream.AddFormField('message', strMessage);
        FormDataStream.AddFormField('subject', strSubject);
        if strBCC <> '' then
          FormDataStream.AddFormField('emailbcc', strBCC);
        if Assigned(slAttachments) then begin
          if (slAttachments.Count = 1) then begin
            FormDataStream.AddFile('upfile', slAttachments[0]);
          end else if (slAttachments.Count > 1) then begin
            raise Exception.Create('Too many attachments defined, only 1 allowed at maximum');
          end;
        end;

        Response := Trim(HTTPClient.Post(strHost + strPHPScript, FormDataStream));
        if Response = 'OK' then begin
          Result := true;
        end else begin
          Raise Exception.Create(Response);
        end;
      finally
        FormDataStream.Free;
      end;
    finally
      HTTPClient.Free;
    end;
  except
    on e: EIdException do begin
      ShowMessage('An error occured while connecting with the internet, please try again later.' + #13 +
        'A firewall or proxy could be blocking the communication. Details:' + #13 +
        e.Message)
    end;
  end;
end;

end.
